﻿using System.Collections.Generic;
using System.Windows;
using ThinkGeo.MapSuite.Core;
using ThinkGeo.MapSuite.WpfDesktopEdition;
using System;
using System.Collections.ObjectModel;

namespace SampleApp_MS3_7583
{
    /// <summary>
    /// Interaction logic for TestWindow.xaml
    /// </summary>
    public partial class TestWindow : Window
    {
        private ManagedProj4Projection proj4;
        LayerOverlay overlay1;
        LayerOverlay overlay;
        public static readonly RectangleShape MaxExtentForWrappingDateline = new RectangleShape(-400767557376, 20026376, 400767557376, -20026376);
        ShapeFileFeatureLayer worldLayer;
        LayerOverlay layerOverlay;
        public const int DotsPerInch = 96;

        public TestWindow()
        {
            InitializeComponent();
        }

        private void Window_Loaded(object sender, RoutedEventArgs e)
        {
            wpfMap1.MapUnit = GeographyUnit.Meter;
            //wpfMap1.CurrentExtent = new RectangleShape(-10000000, 5000000, 10000000, -5000000);
            wpfMap1.BackgroundOverlay.BackgroundBrush = new GeoSolidBrush(GeoColor.FromArgb(255, 198, 255, 255));

            proj4 = new ManagedProj4Projection();
            proj4.InternalProjectionParametersString = ManagedProj4Projection.GetWgs84ParametersString();
            proj4.ExternalProjectionParametersString = ManagedProj4Projection.GetSphericalMercatorParametersString();
            proj4.Open();


            worldLayer = new ShapeFileFeatureLayer(@"..\..\data\Countries02.shp");
            worldLayer.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle = AreaStyles.Country1;
            worldLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;
            worldLayer.FeatureSource.Projection = proj4;

            layerOverlay = new LayerOverlay();
            layerOverlay.WrappingMode = WrappingMode.WrapDateline;

            FileBitmapTileCache bitmapTileCache = new FileBitmapTileCache();
            bitmapTileCache.ClearCache();
            bitmapTileCache.CacheDirectory = AppDomain.CurrentDomain.BaseDirectory + @"Resources\Tiles\";
            bitmapTileCache.CacheId = "Mercator";
            bitmapTileCache.TileAccessMode = TileAccessMode.ReadAddDelete;
            bitmapTileCache.ImageFormat = TileImageFormat.Png;
            layerOverlay.TileCache = bitmapTileCache;


            worldLayer.Open();
            var wrappingExtent = worldLayer.GetBoundingBox();

            layerOverlay.WrappingExtent = wrappingExtent;
            layerOverlay.Layers.Add("worldLayer", worldLayer);
            wpfMap1.Overlays.Add("WorldOverlay", layerOverlay);

            InMemoryFeatureLayer featureLayer = new InMemoryFeatureLayer();
            featureLayer.Name = "featureLayer180";
            featureLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;
            featureLayer.ZoomLevelSet.ZoomLevel01.DefaultPointStyle = PointStyles.City1;
            featureLayer.WrappingMode = WrappingMode.WrapDateline;
            featureLayer.WrappingExtent = wrappingExtent;
            featureLayer.DrawingMarginPercentage = 30;
            FillFeatures180(featureLayer);
            featureLayer.FeatureSource.Projection = proj4;

            overlay = new LayerOverlay();
            overlay.WrappingMode = WrappingMode.WrapDateline;
            overlay.WrappingExtent = wrappingExtent;
            overlay.Layers.Add("featureLayer180", featureLayer);
            wpfMap1.Overlays.Add(overlay);

            InMemoryFeatureLayer featureLayer1 = new InMemoryFeatureLayer();
            featureLayer1.Name = "featureLayer";
            featureLayer1.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;
            featureLayer1.ZoomLevelSet.ZoomLevel01.DefaultPointStyle = PointStyles.Capital1;
            featureLayer1.WrappingMode = WrappingMode.WrapDateline;
            featureLayer1.WrappingExtent = wrappingExtent;
            featureLayer1.DrawingMarginPercentage = 30;
            FillFeatures(featureLayer1);
            featureLayer1.FeatureSource.Projection = proj4;

            overlay1 = new LayerOverlay();
            overlay1.WrappingMode = WrappingMode.WrapDateline;
            overlay1.WrappingExtent = wrappingExtent;
            overlay1.Layers.Add("featureLayer", featureLayer1);
            wpfMap1.Overlays.Add(overlay1);

            SetCustomZoomLevels();
            SetMapExtent();


            wpfMap1.Refresh();
        }

        private void FillFeatures(InMemoryFeatureLayer featureLayer)
        {
            foreach (var point in GetTestPoints())
            {
                //featureLayer.InternalFeatures.Add(new Feature(proj4.ConvertToExternalProjection(point)));
                featureLayer.InternalFeatures.Add(new Feature(point));
            }
        }

        private void FillFeatures180(InMemoryFeatureLayer featureLayer)
        {
            foreach (var point in GetTestPoints180())
            {
                //featureLayer.InternalFeatures.Add(new Feature(proj4.ConvertToExternalProjection(point)));
                featureLayer.InternalFeatures.Add(new Feature(point));
            }
        }

        private void SetCustomZoomLevels()
        {

            worldLayer.Open();

            
            double screenWidth = wpfMap1.ActualWidth;
            double screenWidthInMeters = (screenWidth / DotsPerInch) * 0.0254;
            double worldWidth = worldLayer.GetBoundingBox().Width;
            double scaleMax = GetScaleFromResolution(worldLayer.GetBoundingBox(), wpfMap1.ActualWidth, wpfMap1.ActualHeight, GeographyUnit.Meter); //worldWidth / screenWidthInMeters;


            for (int i = 0; i < 15; i++)
            {
                wpfMap1.ZoomLevelSet.CustomZoomLevels.Add(new ZoomLevel(scaleMax));
                scaleMax = scaleMax / 2;
            }


            int width = (int)Math.Ceiling(96 * 39.3701 / wpfMap1.ZoomLevelSet.CustomZoomLevels[0].Scale * worldWidth);
            layerOverlay.TileWidth = width;
            layerOverlay.TileHeight = width;
            layerOverlay.TileCache.TileMatrix = new MapSuiteTileMatrix(wpfMap1.ZoomLevelSet.CustomZoomLevels[0].Scale, width, width, wpfMap1.MapUnit);
            layerOverlay.TileCache.TileMatrix.BoundingBox = MaxExtentForWrappingDateline;

        }

       
        public double GetScaleFromResolution(RectangleShape extent, double screenWidthInPixel, double screenHeightInPixel, GeographyUnit unit)
        {

            Dictionary<string, double> inchesPerUnits = new Dictionary<string, double>();
            inchesPerUnits.Add("DecimalDegree", 4374754);
            inchesPerUnits.Add("Feet", 12.0);
            inchesPerUnits.Add("Meter", 39.3701);

            double resolution = extent.Width / screenWidthInPixel;//Math.Max(extent.Width / screenWidthInPixel, extent.Height / screenHeightInPixel);
            double scale = resolution * inchesPerUnits[unit.ToString()] * DotsPerInch;
            return scale;
        }


        private void SetMapExtent()
        {

            RectangleShape rectDegree = new RectangleShape(-140, 80, 140, -75);
            RectangleShape rectMeters = proj4.ConvertToExternalProjection(rectDegree);

            wpfMap1.CurrentExtent = rectMeters; 
            wpfMap1.RestrictExtent = new RectangleShape(-20037376 * 10, 15538711.0963092, 20037376 * 10, -12932243.111992);

        }
       
        private IEnumerable<PointShape> GetTestPoints180()
        {
            yield return new PointShape(150, 40);
            yield return new PointShape(160, 40);
            yield return new PointShape(170, 40);
            yield return new PointShape(180, 40);
            yield return new PointShape(-170, 40);
            yield return new PointShape(-160, 40);
            yield return new PointShape(-150, 40);

        }

        private IEnumerable<PointShape> GetTestPoints()
        {
            yield return new PointShape(0, 40);
            yield return new PointShape(5, 40);
            yield return new PointShape(10, 40);
            yield return new PointShape(15, 40);
            yield return new PointShape(20, 40);
            yield return new PointShape(25, 40);
            yield return new PointShape(30, 40);
            yield return new PointShape(35, 40);
            yield return new PointShape(40, 40);

        }

        private void btnZoomWorks_Click(object sender, RoutedEventArgs e)
        {
            RectangleShape boundingBox = overlay1.GetBoundingBox();
            wpfMap1.CurrentExtent = boundingBox;
            wpfMap1.CenterAt(boundingBox.GetCenterPoint());
        }

        private void btnZoomBug_Click(object sender, RoutedEventArgs e)
        {
           RectangleShape boundingBox= overlay.GetBoundingBox();
           wpfMap1.CurrentExtent = boundingBox;
           wpfMap1.CenterAt(boundingBox.GetCenterPoint());
        }

     

    }
}
